function [M, cost, infeasible] = bintBMatch(W, b, perfect)
% bruteForceBMatch  Solve max-weight (perfect) variable b-matching via BIP
% Same interface as bmatch, but solves a binary integer problem. To check
% correctness.
    
    [iVec, jVec, wVec] = findUT(W);
    nNodes = size(W, 1);
    nEdges = length(iVec);
    
    if length(b) == 1
        b = b * ones(nNodes, 1);
    end
    
    % Reconstitute
    E = sparse(iVec, jVec, 1:nEdges, nNodes, nNodes);
    E = E + E';

    % For each node i, make an indicator row vector i that is 1 on every
    % edge on which i is incident.
    A(nNodes, nEdges) = 0;
    for n = 1:nNodes
        eVec = E(:,n);
        eVec = eVec(eVec ~= 0);
        A(n,eVec) = 1;
    end
    
    opts = optimset('Display', 'Off', 'TolXInteger', 1e-16);    
    if perfect        
        [x, fval, exitflag] = bintprog(-wVec, [], [], A, b, [], opts);
    else
        % Inequality constraint
        [x, fval, exitflag] = bintprog(-wVec, A, b, [], [], [], opts);
    end
    
    if exitflag == -2
        M = [];
        cost = [];
        infeasible = true;
        return;
    else
        infeasible = false;
    end
    
    x = logical(x);
    
    if all(x == 0)
        M = sparse(nNodes, nNodes);
        cost = 0;
    else
        M    = symmSparse(iVec(x), jVec(x), true(sum(x), 1), nNodes);
        cost = sum(wVec(x));        
    end    
end

