function [ good ] = checkBPrivacy(X, G, S, b)
% checkBPrivacy  Verify that solution is b-privaate
%
%   good = checkBPrivacy(X, G, S, b) checks that the solution with
%   compatability graph G and star pattern S ensures that each row in X is
%   b-private (where b is a vector whose length is the rows of X).
%
%   Every k-private solution is also b-private.

    [N, D] = size(X);
        
    % columnize b   
    % (G, S) is b-private is each entry cannot be distinguished from b - 1
    % other entries.
    b = b - 1;               
    if isscalar(b)
        b = b * ones(N, 1);
    else
        b = b(:);
    end
    
    % Check that the compatibility graph G has enough edges
    assert(all(sum(G, 2) >= b));
    assert(all(sum(G, 1) >= b'));
        
    % Check that the compatability graph G is correct
    [iVec, jVec] = find(G);
    S = logical(S);
    for ix = 1:length(iVec)
        % Edge (i,j) implies that rows i and j, after starring, are
        % indistinguishable. That is, all non-starred dimensions should be
        % identical.
        i = iVec(ix);
        j = jVec(ix);
        
        % vector xi is compatible with vector yj if xi(k) = yj(k) for all elements of yj(k) ? ?.
        %
        % WHAT IS THE DISTINCTION BETWEEN I AND J?
        xiSi = X(i,~S(i,:));
        yjSi = X(j,~S(i,:));
        assert(all(xiSi == yjSi));

        % Only would be true in the symmetric case.
%         xiSj = X(i,~S(j,:));
%         yjSj = X(j,~S(j,:));
%         assert(all(xiSj == yjSj));
        
        % All disagreements must be covered by both entries' star patterns.
        %disagree = xor(X(i,:), X(j,:));
        %assert(all( ((S(i,:) | S(j,:)) & disagree) == disagree));
    end
    
    good = true;

end

