function [IDX,B,S]=agglomerative(X,k)
%
% function [IDX,B,S]=agglomerative(X,k)
% 
% Agglomerative clustering ensuring cluster size at least k
% outputs IDX as the same data structure as kmeans,
% B is an adjacency matrix
% S is the star pattern
%
[n,d]=size(X);
S=eye(n,n);
C=[];

iter = 0;

while (sum(sum(S')>0)>1)
  Dlow=1e99;
  ilow=1;
  jlow=2;
  for i=1:n
    ni = sum(S(i,:));
    Xi=X(find(S(i,:)),:);
    if (ni>0)
      for j=i+1:n
        nj = sum(S(j,:));
        if (nj>0)
          Xj=X(find(S(j,:)),:);
          D=(ni+nj)*stars([Xi;Xj])-ni*stars(Xi)-nj*stars(Xj);
          if (D<Dlow)
            Dlow=D;
            ilow=i;
            jlow=j;
          end
        end
      end
    end
  end
  S(ilow,:)=max(S(ilow,:),S(jlow,:));
  S(jlow,:)=zeros(1,n);
  if (sum(S(ilow,:))>=k)
    C=[C;S(ilow,:)];
    S(ilow,:)=zeros(1,n);
  end
  iter = iter + 1;
  fprintf(1, 'Agglomoerative iter = %d\n', iter);
end

% For the last remaining cluster in S, dole out its pieces to closest
% remaining ones in C and get rid of it
i = find(sum(S')>0);
jj = find(S(i,:)>0);

for kk=1:length(jj)
  j=jj(kk);
  Dlow=1e99;
  clow=1;
  for c=1:size(C,1)
    nc=sum(C(c,:));
    D=(nc+1)*stars(X([find(C(c,:)) find(S(j,:))],:))-nc*stars(X(find(C(c,:)),:));
    if (D<Dlow)
      Dlow=D;
      clow=c;
    end
  end
  C(c,j)=1;  
  S(i,j)=0;
end

IDX=zeros(size(C,1),1);
for i=1:n
  IDX(i)=find(C(:,i));
end

B = zeros(n,n);
for i=1:n
  for j=1:n
    if (IDX(i)==IDX(j)) 
      B(i,j)=1;
    end;
  end
end
S=zeros(n,d);
for i=1:n
  for j=1:n
    if (B(i,j)>0.5)
      for dd=1:d
        if (X(i,dd)~=X(j,dd))
          S(i,dd)=1;
        end
      end
    end
  end
end

