function [Gfinal, Sfinal, Wfinal] = gvbm_fake_write(X, kk, outfile, varargin)
% gvbm  General variational b-matching with at least k neighbors
%
%   [Gfinal, Sfinal] = gvbm(X, k, runLog) 
%                    = gvbm(X, k, runLog, 'PropertyName', PropertyValue)
%
%   X is the binary data matrix and k is a scalar or vector of neighbor
%   counts. Prints progress to runLog.
%
%   Output adjacency matrix (between the bipartition, so it may not be
%   symmetric!) in Gfinal and the star pattern in Sfinal.
%
%   Available properties:
%       - Ginit -- initial adjacency matrix. Must be a valid solution.
%       - symmetric (true/false) -- use bipartiter or unipartiter solver
%       - greedy -- whether annealing sticks to the lowest so far or not
%       - negScale, posScale -- annealing schedule; see code
%       - maxIter
%

    p = inputParser;
    p.addRequired('X');
    p.addRequired('k', @isnumeric);
    p.addRequired('outfile', @ischar);
    p.addParamValue('Ginit', []);
    p.addParamValue('Winit', []);
    p.addParamValue('tempInit', -1);
    p.addParamValue('symmetric', false);
    p.addParamValue('greedy', false);
    p.addParamValue('negScale', 2);
    p.addParamValue('posScale', 0.92);
    p.addParamValue('maxIter', 200);
    p.addParamValue('metropolis', false);
    p.addParamValue('roundMe', true);
    p.parse(X, kk, outfile, varargin{:});
    
    o = p.Results;
    
    [N, D] = size(X);

    if isempty(o.Winit)
        % Initialize W to 1... but normalize here.    
        W = ones(N, D);
        % Maybe remove this line for the big datasets? On the other hand,
        % unimatch converges, so might as well keep it.
        W = W / sum(W(:)); 
    else
        W = o.Winit;
    end
    
    Wfinal = W;

    Sold = N*D;    
    if ~isempty(o.Ginit)
        G    = o.Ginit;
        Gold = o.Ginit;
    else        
        Gold = zeros(N,N);
    end


    Sfinal = ones(size(X));
    Strail = [];

    maxK = max(kk);
    fprintf(1,  'n=%d d=%d maxk=%d\n', N, D, maxK);

    temperature = o.tempInit; % negative value just means (steal it).
    
    GW(N,N) = 0;

    for i = 1:N
        for j = 1:N
            GW(i,j) = sum(W(i,:) .* (X(i,:)~=X(j,:)));
        end
    end

    %GW = GW + rand(size(GW)) * 1e-6;

    % Special case: If we start with an initial value, we skip the
    % first iteration
    %
    % Probably a smarter way... but this works!
    if iter > 1 || isempty(o.Ginit)        
        % Drop the excesses        
        if o.symmetric
            GW = triu(GW);
            Gc = double(bmatch_write_fake(GW, N - kk, false, false, o.roundMe));                
        else
            % CODE TO CORRECT DO IMPERFECT BIPARTITE B-MATCHING... EITHER
            % DO THE LP (Gurobi?) AND DOUBLE-CHECK CONVERGENCE.
            %
            % I think Gurobi crashes when you try to run in two MATLAB
            % processes? Usually in the checkusername routine...
            % Double-check the licensing?

            %Gc = bi_bmatch(GW, N - 1 - kk, false, false, true); % Gurobi needed for accuracy
            error('We dont do bi_bmatch anymore');            
        end        
    end
end
