function script_2(fi, ik, varargin)

    p = inputParser;
    p.addRequired('fi', @isscalar);
    p.addRequired('ik', @isscalar);    
    p.addParamValue('tempInit', 1);
    p.addParamValue('vbTempInit', 1);
    p.addParamValue('warmStart', true);
    p.addParamValue('roundMe', false);
    
    p.parse(fi, ik, varargin{:});
    o = p.Results;
    
    dbstop if error;

    files{1}='bupa.bin';
    files{2}='wine.bin';
    files{3}='heart.bin';
    files{4}='ecoli.bin';
    files{5}='hepatitis.bin';
    files{6}='wdbc.mat';
    files{7}='auto-mpg.mat';
    files{8}='forestfires.mat';    

    kkVec = 20:-6:1;
    kk = kkVec(ik);

    %% Single fi iteration
    if fi > 5
        ss = load(files{fi});
        dataS = double(ss.data);
    else
        dataS=load(files{fi});
    end

    [N, D] = size(dataS);
    if mod(N, 2) == 1 % odd
        dataS(end,:) = [];
    end

    %% Begin with agglomerative, which we load
    s = sprintf('uci_results/runagg_8set%d.%d.mat', fi, kk);
    agg = load(s);

    % Guarantee improvement. After all, it's a relaxation.
    % Setting the temperature to 1 allows us to use the optimal
    % setting.
    
    if o.warmStart
        Gi = agg.B2;
    else
        Gi = [];
    end
    
    [B, S] = gvbm(dataS, kk, 'gvbm_uci_uni_runlog_2', 'symmetric', true, 'tempInit', o.tempInit, 'Ginit', Gi, 'roundMe', o.roundMe);
    s = sprintf('uci_results/rununi_8set_2_%d.%d.mat', fi, kk);
    save(s, 'B', 'S');
    uniStars = sum(S(:));

    [B, S] = gvbm(dataS, kk, 'gvbm_uci_vb_runlog_2', 'tempInit', o.vbTempInit, 'Ginit', B);
    s = sprintf('uci_results/run_8set_2_%d.%d.mat', fi, kk);
    save(s, 'B', 'S');
    vbStars = sum(S(:));
        
    out = sprintf('starlog_%d.%d', fi, ik);
    fid = fopen(out, 'a');        
    for io = [1 fid]
        fprintf(io, '*** OPTIONS ***\n');
        fprintf(io, toString(o, 'disp'));
        fprintf(io, '*** STAR SUMMARY ***\n');
        fprintf(io, 'agg: %d, uni: %d, vb: %d\n', sum(agg.S2(:)), uniStars, vbStars);
    end
    fclose(fid);
    
end
