function [Gfinal,Sfinal]=unimatching(X,kkk,epsilon,plot_diagnostic)
%
% function [Gfinal,Sfinal]=unimatching(X,kkk,epsilon)
% 
% Bmatching with at least k connections
% epsilon controls the variational bound
% outputs G is an adjacency matrix
% outputs S is the star pattern
%
%
% TODO: Change the call to edmondsreduction.

if (nargin<3)
  epsilon=1E-5;
end

if nargin < 4
    plot_diagnostic = false;
end

[n,d]=size(X);

W=ones(n,d);
W=W/sum(sum(W));

Sold = n*d;
Snew = Sold-1;
costos=[];
costo=Sold;
costn=Snew;
Strail = round(rand(1,20)*10000);

zzz = 1;
maxzzz=80;

Gold=zeros(n,n);

fid = fopen('runlog','a');

Sfinal = ones(size(X));

fprintf(1,'n=%d d=%d mean(k)=%d\n',n,d,mean(kkk)); fprintf(fid,'n=%d d=%d mean(k)=%d\n',n,d,mean(kkk));

pattern = ones(n,n) - diag(ones(n,1));

sumSHist(maxzzz-1) = 0;

while (zzz<maxzzz)

  costo=costn;
  
  D=zeros(n,n);
  for i=1:n
    for j=1:n
      D(i,j)=sum(W(i,:).*(X(i,:)~=X(j,:)));
    end
  end
  assert(all(vec(D == D')));
  
  % false -> imperfect
  % true  -> verbose
  
  % bmatch finds a MAXIMUM weight matching with <= b constraint.
  % We want to solve a MINIMUM weight matching with >= b constraint.
  % So negate the weights and relative complement the result.
    
  bVec = kkk - 1;
  %G = bmatch(-D, bVec, false, true);  
  
  % This is a minimization problem... Hmm, you might have spilled the beans
  compB = n - bVec - 1;
  [GPerf, cPerf] = bmatch(D, compB, true,  false);
  [G, c]        = bmatch(D, compB, false, false);
  assert(cPerf <= c);
  
  perfCost = sum(vec(D(GPerf)));
  impCost  = sum(vec(D(G)));  
  fprintf(1, 'perfCost = %g ; impCost = %g\n', perfCost, impCost);  
  assert(perfCost <= impCost);  
  
  assert(all(sum(GPerf, 2) == compB));
  assert(all(sum(G, 2)     <= compB));
  
  
  fprintf(1, 'RAW    bmatching nnz = %d; cost = %g ; range(vec(D)) = %g; norm(D) = %g\n', nnz(G), sum(vec(D(G))), range(vec(D ~= 0)), norm(D))

  
  G = logical(pattern - G);
  G = full(G);
  % double-check
  assert(all(sum(G, 2) >= bVec));  
  fprintf(1, 'INVERT bmatching nnz = %d; cost = %g ; range(vec(D)) = %g; norm(D) = %g\n', nnz(G), sum(vec(D(G))), range(vec(D ~= 0)), norm(D))      

  for i=1:n
    G(i,i)=1;
  end
  fprintf(1,'Old value %e ',sum(sum(Gold.*D)));  fprintf(fid,'Old value %e ',sum(sum(Gold.*D)));
  fprintf(1,' new value %e\n',sum(sum(G.*D)));  fprintf(fid,' new value %e\n',sum(sum(G.*D)));
  
  % S is the star matrix
  S=zeros(size(X));
  for i=1:n
    for j=1:n
      if (G(i,j)>0.5)
        for dd=1:d
          if (X(i,dd)~=X(j,dd))
            S(i,dd)=1;
          end
        end
      end
    end
  end

  Snew=sum(sum(S));
  sumSHist(zzz) = Snew;
  if (Snew<sum(sum(Sfinal)))
    Sfinal=S;
    Gfinal=G;
  end

  if (Snew>Sold)
    G=Gold;
    Snew=Sold;
  else    
    Gold=G;
    Sold=Snew;
  end

  costn = 0;
  % n is the annealing factor... it's trying to force convergence.  
  for i=1:n
    for k=1:d
      costn = (1+epsilon)^n-prod(1+epsilon-G(i,:).*(X(i,k)~=X(:,k)'));
      W(i,k) = n*log(1+epsilon)+sum(G(i,:).*(X(i,k)~=X(:,k)'))*log(epsilon/(1+epsilon));
    end
  end
  fprintf(1,'Iter=%d Stars=%d Cost=%e Epsilon=%f\n',zzz,Snew,costn,epsilon);
  fprintf(fid,'Iter=%d Stars=%d Cost=%e Epsilon=%f\n',zzz,Snew,costn,epsilon);
  
  % So this is annealing.
  epsilon=epsilon*0.8;
  zzz=zzz+1;

  %W=W-max(max(W));
  W=exp(W);
%  W=W/(sum(sum(W)));

  Strail = [Strail Snew];

%  if (length(unique(Strail(end-40:end)))<=1)
%    zzz=maxzzz+1;
%  end

end

fclose(fid);

figure;
plot(sumSHist);
