function [Gfinal,Sfinal]=vbmatching(X,kkk,epsilon)
%
% function [Gfinal,Sfinal]=vbmatching(X,kkk,epsilon)
%
% Epsilon-based vbmatching with descent-only path and early (40-iteration) stopping
% 
% Bmatching with at least k connections
% epsilon controls the variational bound
% outputs G is an adjacency matrix
% outputs S is the star pattern
%

if (nargin<3)
  epsilon=1E-5;
end

[n,d]=size(X);

if length(kkk) == 1
    kkk = kkk * ones(n,1);
end

assert(length(kkk) == n);

W=ones(n,d);
W=W/sum(sum(W));

Sold = n*d;
Snew = Sold-1;
costos=[];
costo=Sold;
costn=Snew;
Strail = round(rand(1,20)*10000);
G  = round(rand(n,n));

A = zeros(2*n,n*n); b=zeros(2*n,1); Aeq = []; beq = [];
zzz = 1;
maxzzz = 200;

Gold = zeros(n,n);
Sold = n*d;
Sfinal = ones(size(X));

while zzz < maxzzz
  Sold=Snew;
  costo=costn;
  
  D=zeros(n,n);
  for i=1:n
    for j=1:n
      D(i,j)=sum(W(i,:).*(X(i,:)~=X(j,:)));
    end
  end
  D = D+rand(size(D))*1e-4;
  N = n*n;
  f = -reshape(max(max(D))-D,n*n,1);
  for i=1:n
    A1 = zeros(1,N);
    for j=1:n
      A1(n*(i-1)+j)=1;
    end
    A(i,:) = A1;
    b(i) =  kkk(i);
  end
  for i=1:n
    A1 = zeros(1,N);
    for j=1:n
      A1(n*(j-1)+i)=1;
    end
    A(i+n,:) = A1;
    b(i+n) = kkk(i);
  end
%  tic;
%  x=linprog(f,A,b,Aeq,beq,zeros(size(f)),ones(size(f)));
%  fprintf(1, 'MATLAB linprog took %g secs\n', toc);

%  tic;
  x = linprog(f,A,b,Aeq,beq,zeros(size(f)),ones(size(f)));
%  fprintf(1, 'Gurobi linprog took %g secs\n', toc);

%  assert(all(x == x_gurobi));

  G  = zeros(n,n);
  for i=1:n
    for j=1:n
      G(i,j)=x(n*(j-1)+i);
    end
  end
  S=zeros(size(X));
  G=round(G);
  for i=1:n
    for j=1:n
      if (G(i,j)>0.5)
        for dd=1:d
          if (X(i,dd)~=X(j,dd))
            S(i,dd)=1;
          end
        end
      end
    end
  end

  Snew=sum(sum(S));
  costn = n*d;
  for i=1:n
    for k=1:d
     costn = costn-prod(1+epsilon-G(i,:).*(X(i,k)~=X(:,k)'));
      W(i,k) = n*log(1+epsilon)+sum(G(i,:).*(X(i,k)~=X(:,k)'))*log(epsilon/(1+epsilon));
    end
  end
  fprintf(1,'Iter=%d Stars=%d Cost=%f Epsilon=%f\n',zzz,Snew,costn,epsilon);
  epsilon=epsilon*0.9;
  zzz=zzz+1;
  % costos=[costos; Snew costo]; plot(costos); pause(0.0001);
  W=W-max(max(W));
  W=exp(W);
  W=W/(sum(sum(W)));
  if (Snew<sum(Sfinal(:)))
    Sfinal=S;
    Gfinal=G;
  end

    % Don't move to a higher energy state!
    if (Snew>Sold)
        G=Gold;
        Snew=Sold;
    else
        Gold=G;
        Sold=Snew;
    end

  Strail = [Strail Snew];

  % Early stopping
  if zzz > 40 && length(unique(Strail(end-10:end)))>1
      zzz = maxzzz + 1;
  end
end

