function [x, r, c] = minPdist(D)
% argminPdist  Return rectangular coordinates from pdist
% 
% [x,r,c] = argminPdst(D) returns the minimum and its coordinates (row, col)

    % From the manual: D is a row vector of length m(m?1)/2,
    % corresponding to pairs of observations in X. The distances are
    % arranged in the order (2,1), (3,1), ..., (m,1), (3,2), ..., (m,2), ..., (m,m?1))
    N = length(D);
    % Quadratic formula; triangular root
    m = 0.5 + sqrt(1 + 8*N) / 2;
    
    % Break ties
    perturb = 1e-6*rand(size(D));
    [x, i] = min(D + perturb);
    x = x - perturb(i);
    
    % http://stackoverflow.com/questions/8131071/mapping-a-triangular-matrix-to-vector
    %CI = m - round( sqrt( 2*(1 + N - I) ) );
    %RI = mod(I + CI.*(CI+1)/2 - 1, m) + 1;    
    
    % Simplified to calculate just one
    c = m - round(sqrt(2*(1 + N - i)));
    r = mod(i + c * (c + 1) / 2 - 1, m) + 1;
end


