%% Path
addpath blossom
addpath experiment

%% Run
nTrials = 10000;
nNodes  = 8;
deg     = 4; % allow natural zero-weights

testImperfect = true;

ps = '++';
for t = 1:nTrials
    %% Run one trial
    adj = randGraph(nNodes, deg);
        
    W   = abs(sprandn(adj));
    %W   = ceil(10000 * sprand(adj));
    % Symmetrize
    W   = triu(W) + triu(W)';
    %W   = full(W);
    
    % Also randomly generate b
    degVec = full(sum(W ~= 0, 2));
    b   = arrayfun(@(d) randi(d, 1), degVec);    
    %b = min(2 * ones(size(W, 1), 1), degVec);
    
    [p2p, c2p, i2p] = bintBMatch(W, b, true);    
    % If the problem is not feasible, then bmatch may segfault. We don't
    % want that, because now we're running a mex file (and would segfault
    % the whole thing). So we skip the iteration.    
    if i2p
        fprintf(1, 'I');
    else        
        [p1p, c1p, i1p] = bmatch(W, b, true);    

        % Check b (sanity)
        if ~i1p
            assert(all(full(sum(p1p, 2)) == b));
            assert(all(full(sum(p2p, 2)) == b));            
        end    
        % Both must be feasible, or infeasible
        assertEqual(i1p, i2p);
        assertElementsAlmostEqual(c1p, c2p);
    end
        
    if testImperfect
        [p2, c2, i2] = bintBMatch(W, b, false);
        
        if i2
            fprintf(1, 'i');
        else
            fprintf(1, 'i'); % Lowercase i: imperfect is infeasible.
            [p1, c1, i1] = bmatch(W, b, false);  
            % Check b (sanity)
            if ~i1
                assert(all(full(sum(p1, 2)) <= b));
                assert(all(full(sum(p2, 2)) <= b));                        
            end
            
            if any(full(sum(p2, 2)) < b)
                disp('Imperfection attained.');
            end            

            assertEqual(i1, i2);
            assertElementsAlmostEqual(c1, c2);
            diff(t) = c1 - c2;
            dist(t) = full(sum(abs(vec(p1 - p2))));

            if ~i2p && ~i2
                % The perfect should lower-bound imperfect
                assert(c1 >= c1p);
                assert(c2 >= c2p);
            end
        end    
    else
        i2 = false;
    end
    
    % Success!
    nSucc = ~i2 + ~i2p;
    fprintf(1, ps(1:nSucc));    
    
    if mod(t, 40) == 0
        fprintf(1, '\n');
    end    
end

figure;
hist(diff, 100);
title('b-matching suboptimality');

figure;
hist(dist, 100);
title('number of edge changes');
